"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _crypto = _interopRequireDefault(require("crypto"));

var _fields = require("@commodo/fields");

var _commodoFieldsObject = require("commodo-fields-object");

var _validation = require("@webiny/validation");

var _dbArgs = _interopRequireDefault(require("./dbArgs"));

var _mdbid = _interopRequireDefault(require("mdbid"));

var _apiSecurity = require("@webiny/api-security");

var _handlerGraphql = require("@webiny/handler-graphql");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const APIKeyModel = (0, _fields.withFields)({
  name: (0, _fields.string)({
    validation: _validation.validation.create("required")
  }),
  description: (0, _fields.string)({
    validation: _validation.validation.create("required")
  }),
  permissions: (0, _commodoFieldsObject.object)({
    list: true,
    value: []
  })
})();

const generateToken = (tokenLength = 48) => {
  const token = _crypto.default.randomBytes(Math.ceil(tokenLength / 2)).toString("hex"); // API Keys are prefixed with a letter "a" to make token verification easier.
  // When authentication plugins kick in, they will be able to tell if they should handle the token by
  // checking the first letter and either process the token or skip authentication completely.


  if (token.startsWith("a")) {
    return token;
  }

  return `a${token.slice(0, tokenLength - 1)}`;
};

var _default = context => {
  const {
    db,
    security,
    tenancy
  } = context;
  return {
    async getApiKeyByToken(token) {
      const tenant = tenancy.getCurrentTenant();
      const [[apiKey]] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          GSI1_PK: `T#${tenant.id}`,
          GSI1_SK: `API_KEY#${token}`
        }
      }));
      return apiKey;
    },

    async getApiKey(id) {
      // Check if it's an ID or an actual API key (API keys start with a letter "a")
      const tenant = tenancy.getCurrentTenant();
      const permission = await security.getPermission("security.apiKey");

      if (!permission) {
        throw new _apiSecurity.NotAuthorizedError();
      }

      const [[apiKey]] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: `T#${tenant.id}`,
          SK: `API_KEY#${id}`
        }
      }));
      return apiKey;
    },

    async listApiKeys() {
      const tenant = tenancy.getCurrentTenant();
      const permission = await security.getPermission("security.apiKey");

      if (!permission) {
        throw new _apiSecurity.NotAuthorizedError();
      }

      const [apiKeys] = await db.read(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: `T#${tenant.id}`,
          SK: {
            $beginsWith: "API_KEY#"
          }
        }
      }));
      return apiKeys;
    },

    async createApiKey(data) {
      const identity = security.getIdentity();
      const tenant = tenancy.getCurrentTenant();
      const permission = await security.getPermission("security.apiKey");

      if (!permission) {
        throw new _apiSecurity.NotAuthorizedError();
      }

      await new APIKeyModel().populate(data).validate();

      const apiKey = _objectSpread({
        id: (0, _mdbid.default)(),
        token: generateToken(),
        createdBy: {
          id: identity.id,
          displayName: identity.displayName,
          type: identity.type
        },
        createdOn: new Date().toISOString()
      }, data);

      await db.create(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        data: _objectSpread({
          PK: `T#${tenant.id}`,
          SK: `API_KEY#${apiKey.id}`,
          GSI1_PK: `T#${tenant.id}`,
          GSI1_SK: `API_KEY#${apiKey.token}`,
          TYPE: "security.apiKey"
        }, apiKey)
      }));
      return apiKey;
    },

    async updateApiKey(id, data) {
      const tenant = tenancy.getCurrentTenant();
      const permission = await security.getPermission("security.apiKey");

      if (!permission) {
        throw new _apiSecurity.NotAuthorizedError();
      }

      const model = new APIKeyModel().populate(data);
      await model.validate();
      const changedData = await model.toJSON({
        onlyDirty: true
      });
      const apiKey = await this.getApiKey(id);

      if (!apiKey) {
        throw new _handlerGraphql.NotFoundError(`API key "${id}" was not found!`);
      }

      await db.update(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: `T#${tenant.id}`,
          SK: `API_KEY#${apiKey.id}`
        },
        data: changedData
      }));
      return Object.assign({}, apiKey, changedData);
    },

    async deleteApiKey(id) {
      const tenant = tenancy.getCurrentTenant();
      const permission = await security.getPermission("security.apiKey");

      if (!permission) {
        throw new _apiSecurity.NotAuthorizedError();
      }

      await db.delete(_objectSpread(_objectSpread({}, _dbArgs.default), {}, {
        query: {
          PK: `T#${tenant.id}`,
          SK: `API_KEY#${id}`
        }
      }));
      return true;
    }

  };
};

exports.default = _default;
//# sourceMappingURL=apiKey.crud.js.map