"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _responses = require("@webiny/handler-graphql/responses");

var _GraphQLSchemaPlugin = require("@webiny/handler-graphql/plugins/GraphQLSchemaPlugin");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const createDefaultGroups = async context => {
  const tenant = context.tenancy.getCurrentTenant();
  let anonymousGroup = null;
  let fullAccessGroup = null;
  const groups = await context.security.groups.listGroups(tenant);
  groups.forEach(group => {
    if (group.slug === "full-access") {
      fullAccessGroup = group;
    }

    if (group.slug === "anonymous") {
      anonymousGroup = group;
    }
  });

  if (!fullAccessGroup) {
    fullAccessGroup = await context.security.groups.createGroup(tenant, {
      name: "Full Access",
      description: "Grants full access to all apps.",
      system: true,
      slug: "full-access",
      permissions: [{
        name: "*"
      }]
    });
  }

  if (!anonymousGroup) {
    anonymousGroup = await context.security.groups.createGroup(tenant, {
      name: "Anonymous",
      description: "Permissions for anonymous users (public access).",
      system: true,
      slug: "anonymous",
      permissions: []
    });
  }

  return {
    fullAccessGroup,
    anonymousGroup
  };
};

var _default = new _GraphQLSchemaPlugin.GraphQLSchemaPlugin({
  typeDefs:
  /* GraphQL */
  `
        input SecurityInstallInput {
            firstName: String!
            lastName: String!
            login: String!
        }

        extend type SecurityQuery {
            "Get installed version"
            version: String
        }

        extend type SecurityMutation {
            "Install Security"
            install(data: SecurityInstallInput!): SecurityBooleanResponse
        }
    `,
  resolvers: {
    SecurityQuery: {
      version: async (root, args, context) => {
        return await context.security.system.getVersion();
      }
    },
    SecurityMutation: {
      install: async (root, args, context) => {
        const rootTenant = await context.tenancy.getRootTenant();

        if (rootTenant) {
          return new _responses.ErrorResponse({
            code: "SECURITY_INSTALL_ABORTED",
            message: "Security is already installed."
          });
        }

        const {
          data
        } = args;

        try {
          // Create root tenant
          const tenant = await context.tenancy.createTenant({
            id: "root",
            name: "Root",
            parent: null
          });
          context.tenancy.setCurrentTenant(tenant); // Create default groups

          const {
            fullAccessGroup
          } = await createDefaultGroups(context);

          try {
            // Create new user
            await context.security.users.createUser(_objectSpread(_objectSpread({}, data), {}, {
              group: fullAccessGroup.slug
            }), {
              auth: false
            });
          } catch (e) {
            await context.tenancy.deleteTenant("root");
            return new _responses.ErrorResponse({
              code: "SECURITY_INSTALL_ABORTED",
              message: e.message
            });
          } // Store app version


          await context.security.system.setVersion(context.WEBINY_VERSION);
          return new _responses.Response(true);
        } catch (e) {
          return new _responses.ErrorResponse(e);
        }
      }
    }
  }
});

exports.default = _default;
//# sourceMappingURL=install.gql.js.map