"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _crypto = _interopRequireDefault(require("crypto"));

var _responses = require("@webiny/handler-graphql/responses");

var _GraphQLSchemaPlugin = require("@webiny/handler-graphql/plugins/GraphQLSchemaPlugin");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const generateToken = (tokenLength = 48) => {
  const token = _crypto.default.randomBytes(Math.ceil(tokenLength / 2)).toString("hex").slice(0, tokenLength - 1); // Personal access tokens are prefixed with a letter "p" to make token verification easier.
  // When authentication plugins kick in, they will be able to tell if they should handle the token by
  // checking the first letter and either process the token or skip authentication completely.


  return `p${token}`;
};

var _default = new _GraphQLSchemaPlugin.GraphQLSchemaPlugin({
  typeDefs:
  /* GraphQL */
  `
        type SecurityPersonalAccessToken {
            id: ID
            name: String
            token: String
            createdOn: DateTime
        }
        input SecurityPersonalAccessTokenInput {
            name: String
        }

        type SecurityPersonalAccessTokenCreate {
            pat: SecurityPersonalAccessToken
            # The full token - you only receive it once!
            token: String
        }

        type SecurityPersonalAccessTokenCreateResponse {
            data: SecurityPersonalAccessTokenCreate
            error: SecurityError
        }

        type SecurityPersonalAccessTokenResponse {
            data: SecurityPersonalAccessToken
            error: SecurityError
        }

        extend type SecurityUser {
            personalAccessTokens: [SecurityPersonalAccessToken]
        }

        extend type SecurityMutation {
            createPAT(
                data: SecurityPersonalAccessTokenInput!
            ): SecurityPersonalAccessTokenCreateResponse

            updatePAT(
                id: ID!
                data: SecurityPersonalAccessTokenInput!
            ): SecurityPersonalAccessTokenResponse

            deletePAT(id: ID!): SecurityBooleanResponse
        }
    `,
  resolvers: {
    SecurityUser: {
      personalAccessTokens(user, args, context) {
        // A user can only access his own tokens!
        if (user.login !== context.security.getIdentity().id) {
          return null;
        }

        return context.security.users.listTokens(user.login);
      }

    },
    SecurityPersonalAccessToken: {
      token: pat => {
        return pat.token.substr(-4);
      }
    },
    SecurityMutation: {
      createPAT: async (root, {
        data
      }, context) => {
        const identity = context.security.getIdentity();

        if (!identity) {
          throw new Error("Not authorized!");
        }

        try {
          const token = generateToken();
          const tokenData = await context.security.users.createToken(identity.id, _objectSpread(_objectSpread({}, data), {}, {
            token
          }));
          return new _responses.Response({
            pat: tokenData,
            token
          });
        } catch (e) {
          return new _responses.ErrorResponse({
            code: e.code,
            message: e.message,
            data: e.data || null
          });
        }
      },
      updatePAT: async (root, {
        id,
        data
      }, context) => {
        const identity = context.security.getIdentity();

        if (!identity) {
          throw new Error("Not authorized!");
        }

        try {
          const existingToken = await context.security.users.getPersonalAccessToken(identity.id, id);

          if (!existingToken) {
            return new _responses.NotFoundResponse(`PAT "${id}" was not found!`);
          }

          const updatedData = await context.security.users.updateToken(identity.id, id, data);
          return new _responses.Response(Object.assign({}, existingToken, updatedData));
        } catch (e) {
          return new _responses.ErrorResponse({
            code: e.code,
            message: e.message,
            data: e.data || null
          });
        }
      },
      deletePAT: async (root, {
        id
      }, context) => {
        const identity = context.security.getIdentity();

        if (!identity) {
          throw new Error("Not authorized!");
        }

        try {
          const existingToken = await context.security.users.getPersonalAccessToken(identity.id, id);

          if (!existingToken) {
            return new _responses.NotFoundResponse(`PAT "${id}" was not found!`);
          }

          await context.security.users.deleteToken(identity.id, id);
          return new _responses.Response(true);
        } catch (e) {
          return new _responses.ErrorResponse({
            code: e.code,
            message: e.message,
            data: e.data || null
          });
        }
      }
    }
  }
});

exports.default = _default;
//# sourceMappingURL=pat.gql.js.map